"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>
"""
from defence360agent.api.integration_conf import IntegrationConfig
from defence360agent.contracts.config import UIRole
from defence360agent.subsys.panels.generic.panel import get_admin_list


class PamAuth:
    DEFAULT_AUTH_SERVICE = 'system-auth'

    def authenticate(self, username, password) -> bool:
        from pam import pam
        try:
            config = IntegrationConfig().to_dict()
            service = config['PAM']['SERVICE_NAME']
        except KeyError:
            service = self.DEFAULT_AUTH_SERVICE

        p = pam()
        return p.authenticate(
            username,
            password,
            service=service
        )

    async def get_user_type(self, username: str) -> UIRole:
        admins = await get_admin_list()
        return UIRole.ADMIN if username in admins else UIRole.CLIENT
