"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>
"""
from logging import getLogger
from typing import Any, List

import defence360agent.subsys.panels.hosting_panel as hp
from defence360agent.feature_management.constants import (AV, AV_REPORT, FULL,
                                                          NA)
from defence360agent.feature_management.rpc.endpoints.utils import \
    builtin_feature_management_only
from defence360agent.feature_management.utils import (update_default,
                                                      update_users)
from defence360agent.rpc_tools.lookup import RootEndpoints, bind

logger = getLogger(__name__)


@builtin_feature_management_only
class FeatureManagementUpdateEndpoints(RootEndpoints):
    @staticmethod
    async def _update(feature: str, users: List[str], value: Any):
        if not users:
            return {
                'items': 'succeed'
                         if await update_default(feature, value)
                         else 'failed'
            }
        return {'items': await update_users(
            feature, users, value, await hp.HostingPanel().get_users())}

    def _adapt_value(self, feature, value):
        if value:
            return FULL
        if feature == AV and not value:
            return AV_REPORT
        return NA

    @bind('feature-management', 'enable')
    async def feature_management_enable(self, feature: str, users=None):
        """Enable specified feature"""
        return await self._update(
            feature, users, self._adapt_value(feature, True))

    @bind('feature-management', 'disable')
    async def feature_management_disable(self, feature: str, users=None):
        """Disable specified feature"""
        return await self._update(
            feature, users, self._adapt_value(feature, False))
