"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>
"""
import logging
import os
from pathlib import Path

from defence360agent.utils import os_version, OsReleaseInfo

logger = logging.getLogger(__name__)

TEST_REPO_PATH = Path('/etc/yum.repos.d/imunify360-testing.repo')

CHECKSITE = "https://repo.imunify360.cloudlinux.com/defense360"
RPM_KEY = '{}/RPM-GPG-KEY-CloudLinux'.format(CHECKSITE)

# disabled by default
TEMPLATE_REPO = r"""
[imunify360-testing]
name=EL-{version} - Imunify360
baseurl={CHECKSITE}/el/{version}/updates-testing/x86_64/
username=defense360
password=nraW!F@\$x4Xd6HHQ
enabled=0
gpgcheck=1
gpgkey={RPM_KEY}
"""


def install_repo(version):
    if version in (6, 7):
        if not TEST_REPO_PATH.exists():
            TEST_REPO_PATH.write_text(
                TEMPLATE_REPO.format(version=version,
                                     CHECKSITE=CHECKSITE,
                                     RPM_KEY=RPM_KEY))
    else:
        logger.info("Version {} is not supported".format(version))


def migrate(migrator, database, fake=False, **kwargs):
    if fake:
        return

    try:
        if OsReleaseInfo.id_like() & OsReleaseInfo.RHEL_FEDORA_CENTOS:

            version = None
            full_version = os_version()
            if full_version.startswith('6'):
                version = 6
            elif full_version.startswith('7'):
                version = 7

            install_repo(version)
    except Exception as e:
        logger.warning("Unable to add imunify360-testing repo: %s", e)


def rollback(migrator, database, fake=False, **kwargs):
    if fake:
        return

    try:
        os.remove(TEST_REPO_PATH)
    except Exception as e:
        logger.warning(str(e))
