"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>
"""
from logging import getLogger

from defence360agent.utils import run_coro
from defence360agent.utils import antivirus_mode

logger = getLogger(__name__)


@antivirus_mode.skip
def migrate(migrator, database, fake=False, **kwargs):
    try:
        from im360.subsys.panels.plesk import Plesk
        from im360.subsys.panels.plesk.mod_security import ModSecSettings
        if (
            fake
            or not Plesk.is_installed()
            or not run_coro(Plesk.installed_modsec())
        ):
            return
        ModSecSettings.include_modsec_conf()

        from defence360agent.subsys.web_server import graceful_restart_sync
        graceful_restart_sync()
    except Exception as e:
        logger.warning('Error during web-server update: %s', str(e))


@antivirus_mode.skip
def rollback(migrator, database, fake=False, **kwargs):
    try:
        if (
            fake
            or not Plesk.is_installed()
            or not run_coro(Plesk.installed_modsec())
        ):
            return
        ModSecSettings.revert_conf_include()

        from defence360agent.subsys.web_server import graceful_restart_sync
        graceful_restart_sync()
    except Exception as e:
        logger.warning('Error during web-server update: %s', str(e))
