"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>
"""
import logging

import peewee as pw


logger = logging.getLogger(__name__)


def migrate(migrator, database, fake=False, **kwargs):
    """Write your migrations here."""

    class DisabledRule(pw.Model):

        class Meta:
            db_table = 'disabled_rules'
            indexes = (
                (('plugin', 'rule_id'), True),
            )

        id = pw.PrimaryKeyField()
        plugin = pw.CharField(null=False)
        rule_id = pw.CharField(null=False)
        name = pw.TextField(null=False)

    class DisabledRuleDomain(pw.Model):

        disabled_rule_id = pw.ForeignKeyField(
            DisabledRule,
            related_name='domains',
            on_delete='CASCADE')
        domain = pw.CharField(null=False)

        class Meta:
            db_table = 'disabled_rules_domains'
            primary_key = pw.CompositeKey('disabled_rule_id', 'domain')

    migrator.create_model(DisabledRule)
    migrator.create_model(DisabledRuleDomain)


def rollback(migrator, database, fake=False, **kwargs):
    """Write your rollback migrations here."""
    migrator.remove_model(migrator.orm['disabled_rules_domains'])
    migrator.remove_model(migrator.orm['disabled_rules'])
