"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>


Migrate backup config from user oriented config file
to the separate internal file
"""
import os
from typing import Optional

from defence360agent.contracts.config import (
    BackupConfig,
    IConfig,
    IConfigFile,
    LocalConfig,
)
from defence360agent.utils import antivirus_mode


@antivirus_mode.skip
def migrate(
    migrator,
    database,
    fake=False,
    config_file: IConfig = LocalConfig(),
    backup_config_file: Optional[IConfigFile] = None,
    **kwargs,
):
    if fake:
        return
    if backup_config_file is None:
        backup_config_file = BackupConfig()

    if not (config_from := config_file.config_to_dict()):
        return

    # Do not overwrite existing config file
    if os.path.exists(backup_config_file.path):
        return
    backup_conf_current = config_from.get('BACKUP_RESTORE', {})
    config_to = {
        'BACKUP_SYSTEM': {
            'enabled': backup_conf_current.pop('enabled', False),
            'backup_system': backup_conf_current.pop('backup_system', None)
        }
    }
    backup_config_file.dict_to_config(
        config_to, overwrite=True, validate=False
    )
    config_file.dict_to_config(config_from, overwrite=True, validate=False)


@antivirus_mode.skip
def rollback(migrator, database, fake=False, **kwargs):
    """Write your rollback migrations here."""
    pass
