"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>
"""
import peewee as pw


class Country(pw.Model):
    """
    Contains country code and name
    """
    id = pw.CharField(primary_key=True, null=False)
    code = pw.CharField(max_length=2, unique=True, null=False)
    name = pw.CharField(null=False)

    class Meta:
        db_table = 'country'


class CaptchaStat(pw.Model):
    class Meta:
        db_table = 'captcha_stat'
        primary_key = pw.CompositeKey('event', 'ip', 'country',
                                      'domain', 'timestamp')

    event = pw.TextField(null=False)
    ip = pw.TextField(null=False)
    country = pw.ForeignKeyField(Country, null=True)
    domain = pw.TextField(null=True)
    timestamp = pw.IntegerField(null=False)

    count = pw.IntegerField(null=False)


def migrate(migrator, database, fake=False, **kwargs):
    migrator.create_model(CaptchaStat)


def rollback(migrator, database, fake=False, **kwargs):
    cs = migrator.orm['captcha_stat']
    migrator.remove_model(cs)
