"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>
"""
import logging
import os
import shutil
import subprocess

from defence360agent.utils import antivirus_mode, run_coro

logger = logging.getLogger(__name__)

DISABLED_RULES_CONFIG_DIR = '/etc/apache2/conf.d/'
DISABLED_RULES_CONFIG_FILENAME = 'i360_modsec_disable.conf'


@antivirus_mode.skip
def migrate(migrator, database, fake=False, **kwargs):
    if fake:
        return
    from im360.subsys.panels.cpanel import cPanel
    try:
        if not cPanel.is_installed() or not run_coro(
            cPanel.installed_modsec()
        ):
            return

        hp = cPanel()
        path = os.path.join(hp._get_userdata_dir(),
                            hp.GLOBAL_DISABLED_RULES_CONFIG_FILENAME)

        if os.path.exists(path):
            os.makedirs(DISABLED_RULES_CONFIG_DIR, exist_ok=True)
            shutil.move(path, os.path.join(DISABLED_RULES_CONFIG_DIR,
                                           DISABLED_RULES_CONFIG_FILENAME))

            subprocess.check_call(hp.REBUILD_HTTPDCONF_CMD)
    except Exception as e:
        logger.exception("Failed to delete old rules config with %s", e)


def rollback(migrator, database, fake=False, **kwargs):
    pass
