"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>
"""
import peewee as pw


def migrate(migrator, database, fake=False, **kwargs):

    class RemoteProxyGroup(pw.Model):
        """Groups multiple remote proxies together with common data."""
        MANUAL = 'manual'
        IMUNIFY360 = 'imunify360'
        name = pw.CharField(null=False)
        source = pw.CharField(
            null=False,
            constraints=[pw.Check("source in ('{}', '{}')".format(
                MANUAL, IMUNIFY360
            ))])
        enabled = pw.BooleanField(null=False, default=True)

        class Meta:
            db_table = 'remote_proxy_group'
            indexes = (
                (('name', 'source'), True),
            )

    class RemoteProxy(pw.Model):
        group = pw.ForeignKeyField(RemoteProxyGroup, null=False)
        network = pw.TextField(null=False)

        class Meta:
            db_table = 'remote_proxy'

    migrator.create_model(RemoteProxyGroup)
    migrator.create_model(RemoteProxy)


def rollback(migrator, database, fake=False, **kwargs):
    RemoteProxy = migrator.orm['remote_proxy']
    RemoteProxyGroup = migrator.orm['remote_proxy_group']
    migrator.remove_model(RemoteProxy)
    migrator.remove_model(RemoteProxyGroup)
