"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>
"""
import ipaddress


def migrate(migrator, database, fake=False, **kwargs):
    if fake:
        return

    IgnoreListNew = migrator.orm['ignore_list_new']
    IgnoreList = migrator.orm['ignore_list']
    try:
        from im360.utils.validate import IP
        from im360.utils.net import pack_ip_network
    except ImportError:
        pass
    else:
        with database.atomic():
            # FIXME: after migrating to peewee 3 add .iterator()
            ip_strings = [item["ip"] for item in IgnoreList.select().dicts()]
            ips = set()
            for item in ip_strings:
                try:
                    ip = ipaddress.ip_network(item)
                except ValueError:
                    # malformed ip
                    continue
                ips.add(ip)
            for ip in ips:
                net, mask, version = pack_ip_network(ip)
                IgnoreListNew.create(ip=IP.ip_net_to_string(ip),
                                     network_address=net, netmask=mask,
                                     version=version)

    migrator.sql('DROP TABLE ignore_list')
    migrator.sql("ALTER TABLE ignore_list_new RENAME TO ignore_list")


def rollback(migrator, database, fake=False, **kwargs):
    """Write your rollback migrations here."""
    pass
