"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>
"""
import peewee as pw

from defence360agent.feature_management.constants import NA, FULL, AV_REPORT


def migrate(migrator, database, fake=False, **kwargs):
    permissions_model = migrator.orm['feature_management_permissions']
    migrator.add_fields(
        permissions_model,
        proactive_new=pw.TextField(
            default=FULL,
            null=False,
            constraints=[pw.Check("proactive_new in ('{}','{}')".format(
                NA, FULL))]
        ),
        av=pw.TextField(
            default=AV_REPORT,
            null=False,
            constraints=[pw.Check("av in ('{}','{}','{}')".format(
                NA, AV_REPORT, FULL))]
        )
    )

    migrator.sql('UPDATE feature_management_permissions SET av=? '
                 'WHERE cleanup=1', FULL)
    migrator.sql('UPDATE feature_management_permissions SET av=? '
                 'WHERE cleanup=0', AV_REPORT)

    migrator.sql('UPDATE feature_management_permissions SET proactive_new=? '
                 'WHERE proactive=1', FULL)
    migrator.sql('UPDATE feature_management_permissions SET proactive_new=? '
                 'WHERE proactive=0', NA)

    migrator.remove_fields(permissions_model, 'cleanup', 'proactive')


def rollback(migrator, database, fake=False, **kwargs):
    pass
