"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>
"""
import logging

from defence360agent.utils import run_coro, antivirus_mode


logger = logging.getLogger(__name__)


@antivirus_mode.skip
def migrate(migrator, database, fake=False, **kwargs):
    if fake:
        return

    from im360.subsys.panels.plesk import Plesk
    from im360.subsys.panels.plesk.mod_security import (
        plesk_supports_custom_vendors
    )

    try:
        if Plesk.is_installed() and run_coro(plesk_supports_custom_vendors()):
            panel = Plesk()
            installed_vendors = run_coro(panel.modsec_vendor_list())
            if 'imunify360' in " ".join(installed_vendors):
                run_coro(panel.install_settings())
    except Exception as e:
        logger.warning("Unable to reinstall modsec \"custom\" ruleset: "
                       "%s", e)


def rollback(migrator, database, fake=False, **kwargs):
    pass
