"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>
"""
import logging
from datetime import datetime, timedelta

import peewee as pw

from defence360agent.malwarelib.config import MalwareScanType
from defence360agent.utils import split_for_chunk

logger = logging.getLogger(__name__)

types = (
    MalwareScanType.ON_DEMAND,
    MalwareScanType.REALTIME,
    MalwareScanType.MALWARE_RESPONSE,
    MalwareScanType.BACKGROUND,
    MalwareScanType.RESCAN,
)


def migrate(migrator, database, fake=False, **kwargs):
    MalwareHit = migrator.orm['malware_hits']
    MalwareScan = migrator.orm['malware_scans']

    date = (datetime.now() - timedelta(days=30)).timestamp()

    hits_to_delete = list(
        MalwareHit.select(MalwareHit.id)
        .join(MalwareScan)
        .where(MalwareScan.started < date)
    )

    for chunk in split_for_chunk(hits_to_delete):
        sql, params = MalwareHit.delete().where(MalwareHit.id.in_(chunk)).sql()
        migrator.sql(sql, *params)

    sql, params = MalwareScan.delete().where(MalwareScan.started < date).sql()
    migrator.sql(sql, *params)

    migrator.change_fields(
        MalwareScan,
        type=pw.CharField(
            null=False,
            constraints=[pw.Check('type in {}'.format(types))]
        )
    )


def rollback(migrator, database, fake=False, **kwargs):
    pass
