"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>
"""
import csv
import os
import logging

logger = logging.getLogger(__name__)

PROACTIVE_CSV, PROACTIVE_ENV_CSV = 'proactive.csv', 'proactive_env.csv'
PROACTIVE_SQL = """SELECT
  id, timestamp, ip, ip_int, ip_version, ip_country_id, description, action,
  host, path, url, count, uid, gid, rule_id, rule_name
FROM proactive ORDER BY timestamp DESC LIMIT ?"""
PROACTIVE_ENV_SQL = """
SELECT proactive_env.event_id, proactive_env.name, proactive_env.value
FROM proactive_env INNER JOIN ({}) pa ON proactive_env.event_id=pa.id
""".format(PROACTIVE_SQL)
EXPORT_DIR = "/var/lib/imunify360-php-daemon/export"


def export(database, target_dir, events_num):
    for filename, query in [(PROACTIVE_CSV, PROACTIVE_SQL),
                            (PROACTIVE_ENV_CSV, PROACTIVE_ENV_SQL)]:
        cur = database.execute_sql(query, (events_num,))
        with open(os.path.join(target_dir, filename), 'w',
                  newline='', encoding='utf-8'
                  ) as csvfile:
            csv_writer = csv.writer(csvfile)
            csv_writer.writerows(cur)


def migrate(migrator, database, fake=False, **kwargs):
    if fake:
        return
    try:
        os.makedirs(EXPORT_DIR, exist_ok=True)
        export(database, EXPORT_DIR, 1000)
    except Exception:
        # not critical
        logger.exception("Failed to export proactive defence data")


def rollback(migrator, database, fake=False, **kwargs):
    pass
