"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>
"""
import logging
import os

from defence360agent.contracts.config import ConfigFile, IConfig
from defence360agent.utils import log_error_and_ignore

logger = logging.getLogger(__name__)


def migrate(
    migrator,
    database,
    user_config_dir="/etc/imunify360/user_config",
    config_file: IConfig = ConfigFile(),
    fake=False,
    **kwargs
):
    if fake:
        return

    # Migrate root config
    migrate_config(config_file)

    if not os.path.exists(user_config_dir):
        return

    # Migrate user configs
    for username in os.listdir(user_config_dir):
        migrate_config(ConfigFile(username=username))


@log_error_and_ignore()
def migrate_config(config_file: IConfig):
    config = config_file.config_to_dict(normalize=False)
    if not config:
        return
    malware_settings = config.setdefault('MALWARE_SCANNING', {})

    default_action = malware_settings.get("default_action")
    if default_action == "quarantine":
        malware_settings["default_action"] = "cleanup"
        cleanup_settings = config.setdefault('MALWARE_CLEANUP', {})
        keep_original_files = cleanup_settings.get("keep_original_files_days")
        if keep_original_files is not None and keep_original_files < 180:
            cleanup_settings["keep_original_files_days"] = 180
    elif default_action in ("cleanup_or_quarantine", "delete"):
        malware_settings["default_action"] = "cleanup"
    else:
        return

    config_file.dict_to_config(
        config, overwrite=True, validate=False, normalize=False
    )


def rollback(migrator, database, fake=False, **kwargs):
    pass
