"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>
"""
import time

from peewee import CharField, Check, CompositeKey, IntegerField, Model


class TMPMalwareIgnorePath(Model):
    class Meta:
        db_table = "tmp_malware_ignore_path"
        primary_key = CompositeKey("path", "resource_type")

    CACHE = None

    path = CharField()
    resource_type = CharField(
        null=False, constraints=[Check("resource_type in ('file','db')")]
    )
    added_date = IntegerField(null=False, default=lambda: int(time.time()))


def migrate(migrator, *_, fake=False, **__):
    if fake:
        return

    change_malware_ignore_path_model(migrator)


def change_malware_ignore_path_model(migrator):
    migrator.create_model(TMPMalwareIgnorePath)
    migrator.sql(
        "INSERT INTO tmp_malware_ignore_path(path,added_date,resource_type) "
        "SELECT path,added_date,'file' FROM malware_ignore_path"
    )
    migrator.sql("DROP TABLE malware_ignore_path")
    migrator.sql(
        "ALTER TABLE tmp_malware_ignore_path RENAME TO malware_ignore_path"
    )

    migrator.add_index(migrator.orm["malware_ignore_path"], "resource_type")


def rollback(*_, **__):
    pass
