"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>


Drop constrains for icontact_throttle.message_type, since correlation server
can set any type (DEF-19971).
"""
from peewee import CharField, IntegerField, Model


class IContactThrottle(Model):
    class Meta:
        db_table = "icontact_throttle"

    message_type = CharField(primary_key=True)
    #: The last time we sent a notification about :attr:`message_type`
    timestamp = IntegerField(default=0)


def migrate(migrator, database, fake=False, **kwargs):
    if fake:
        return
    migrator.sql(
        "ALTER TABLE icontact_throttle RENAME TO icontact_throttle_old"
    )
    migrator.create_model(IContactThrottle)
    migrator.sql(
        "INSERT INTO icontact_throttle(message_type,timestamp) "
        "SELECT message_type,timestamp FROM icontact_throttle_old"
    )
    migrator.sql("DROP TABLE icontact_throttle_old")


def rollback(migrator, database, fake=False, **kwargs):
    pass
