"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>
"""
from peewee import CompositeKey, Model, CharField, IntegerField


def migrate(migrator, *_, fake=False, **__):
    if fake:
        return

    icontact_throttle = migrator.orm["icontact_throttle"]

    class TmpIContactThrottle(Model):
        class Meta:
            db_table = "tmp_icontact_throttle"
            primary_key = CompositeKey('message_type', 'user')

        message_type = CharField()
        user = CharField(null=True)
        timestamp = IntegerField(default=0)

    migrator.add_fields(
        icontact_throttle,
        user=CharField(null=True),
    )

    # change the primary key
    migrator.create_model(TmpIContactThrottle)
    migrator.sql(
        "INSERT INTO tmp_icontact_throttle (message_type, user, timestamp) "
        "SELECT message_type, user, timestamp FROM icontact_throttle"
    )
    migrator.sql("DROP TABLE icontact_throttle")
    migrator.sql(
        "ALTER TABLE tmp_icontact_throttle RENAME TO icontact_throttle",
    )


def rollback(migrator, *_, fake=False, **__):
    if fake:
        return

    class TmpIContactThrottle(Model):
        class Meta:
            db_table = "icontact_throttle"

        message_type = CharField(primary_key=True)
        timestamp = IntegerField(default=0)

    migrator.create_model(TmpIContactThrottle)
    migrator.sql(
        "INSERT INTO tmp_icontact_throttle (message_type, timestamp) "
        "SELECT message_type, timestamp FROM icontact_throttle"
    )
    migrator.sql("DROP TABLE icontact_throttle")
    migrator.sql(
        "ALTER TABLE tmp_icontact_throttle RENAME TO icontact_throttle",
    )
