"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>


cPanel plugin listening for config changes.

It disables cPanel native feature management and enables Secure Site
if SECURE_SITE feature got enabled in imunify360 config.
"""
import logging

from defence360agent.contracts.config import SecureSiteConfig
from defence360agent.contracts.messages import MessageType
from defence360agent.contracts.plugins import MessageSink, expect
from defence360agent.feature_management.control import (
    disable_native_feature_management,
)
from defence360agent.secure_site.control import (
    disable_secure_site_extension,
    enable_secure_site_extension,
    is_secure_site_extension_enabled,
)

logger = logging.getLogger(__name__)


class CpanelFeatureManagementSwitch(MessageSink):
    async def create_sink(self, loop):
        """MessageSink method"""

    @expect(MessageType.ConfigUpdate)
    async def config_updated(self, _):
        enabled_in_config = SecureSiteConfig.ENABLED
        extension_installed = await is_secure_site_extension_enabled()
        if enabled_in_config and not extension_installed:
            await disable_native_feature_management()
            await enable_secure_site_extension()
        elif not enabled_in_config and extension_installed:
            # we do not enable native feature management back cause
            # 1. the initial native feature management state is not stored
            # 2. it is considered to be removed
            await disable_secure_site_extension()
