"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>
"""
from logging import getLogger

from defence360agent.api import inactivity
from defence360agent.contracts.config import SimpleRpc
from defence360agent.contracts.plugins import MessageSink
from defence360agent.utils import clip, fail_agent_service, recurring_check

logger = getLogger(__name__)


class IdleTimeOutCheck(MessageSink):
    async def create_sink(self, loop):
        self._loop = loop
        if SimpleRpc.SOCKET_ACTIVATION:
            inactivity.track.reset_timer()
            self._task = loop.create_task(
                recurring_check(
                    period=clip(
                        SimpleRpc.INACTIVITY_TIMEOUT // 5, low=1, high=60
                    ),
                )(self._check_timeout,)()
            )
        else:
            self._task = None

    async def shutdown(self):
        if self._task:
            self._task.cancel()
            # CancelledError is handled by @recurring_check():
            await self._task

    async def _check_timeout(self):
        logger.info("Periodical check %s ", inactivity.track)
        if inactivity.track.is_timeout():
            logger.warning('Shutting down due to inactivity.')
            fail_agent_service()
