"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>
"""
from peewee import CharField, Check, TextField

from defence360agent.model import Model, instance
from defence360agent.secure_site.constants import BASIC, PRO


class SecureSite(Model):
    """Secure-site related settings"""

    class Meta:
        database = instance.db
        db_table = "secure_site"

    #: The username of the end-user, or an empty string for the default value
    #: for all new users.
    user = CharField(unique=True)

    #: Type of SecureSite subscription available to the end-user.
    #: Must be either :obj:`.BASIC` or :obj:`.PRO`.
    subscription_type = TextField(
        null=False,
        constraints=[
            Check("subscription_type in ('{}','{}')".format(BASIC, PRO))
        ],
        default=BASIC,
    )

    @classmethod
    def get_subscription_type(cls, user: str) -> str:
        """Get SecureSite permissions by username"""
        if user is None or user == "root":
            # root
            return PRO

        perm, _ = cls.get_or_create(
            user=user, defaults={"subscription_type": BASIC}
        )
        return perm.subscription_type
