"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>
"""
import threading

from defence360agent.application.determine_hosting_panel import (
    get_hosting_panel,
)
from defence360agent.contracts.config import ANTIVIRUS_MODE
from defence360agent.subsys.panels.base import AbstractPanel

_data = threading.local()
_data.panel = None


def HostingPanel() -> AbstractPanel:
    """
    Set the panel if panel is not set or if it is set to defence360agent class
     in Imunify360.

     - the panel is stored in only one place - in this module (_panel)
     - if we are AV we always use AV panel classes
     - if we are Imunify360 we always use im360 panel classes
    """

    if _data.panel is None:
        _data.panel = get_hosting_panel(
            "defence360agent" if ANTIVIRUS_MODE else "im360"
        )
    return _data.panel
