"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>

High-level file operations."""
import errno
import logging
import shutil

logger = logging.getLogger(__name__)


def rmtree(path, ignore_errors=False, onerror=None, *, max_tries=3):
    """More robust shutil.rmtree.

    Retry on "Directory not empty" race condition:
    https://github.com/ansible/ansible/issues/34335#issuecomment-362995700
    """
    for i in range(1, max_tries + 1):
        try:
            return shutil.rmtree(path, ignore_errors, onerror)
        except OSError as e:
            if i == max_tries or e.errno not in [
                errno.EEXIST,
                errno.ENOTEMPTY,
            ]:
                raise

            # Got "Directory not empty" and attempts are not exhausted yet
            logger.warning("Can't remove %s tree, reason: %s", path, e)
