
"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>
"""
class LineBuffer(object):
    """
    Allows to accumulate data, and than iterate over it getting tokens
    split by line breaks '\n'. If at the end there is no line break,
    the data will sit in the line buffer until more data with line
    break comes in.
    """

    def __init__(self):
        self.buf = ''

    def append(self, data):
        self.buf += data

    def __iter__(self):
        return self

    def __next__(self):
        pos = self.buf.find('\n')
        if pos != -1:
            result = self.buf[0:pos]
            self.buf = self.buf[pos+1:]
            return result
        raise StopIteration

    def clean(self):
        self.buf = ''


class SizeBuffer:
    def __init__(self, size_len=2):
        self._buf = b''
        self._size_len = size_len

    def append(self, data):
        self._buf += data

    def __iter__(self):
        return self

    def __next__(self):
        if not self._buf:
            raise StopIteration
        size = int.from_bytes(self._buf[:self._size_len], 'big')
        if len(self._buf[self._size_len:]) >= size:
            data = self._buf[self._size_len:self._size_len + size]
            self._buf = self._buf[self._size_len + size:]
            return data
        raise StopIteration
