"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>

JSON encoders to help with sending messages to server."""
import json
from ipaddress import IPv4Address, IPv4Network, IPv6Address, IPv6Network

from playhouse.shortcuts import model_to_dict

from defence360agent.model import Model


def ip_net_to_string(net) -> str:
    """
        IPv4Network('192.168.1.1/32') -> '192.168.1.1'
        IPv4Network('192.168.1.0/24') -> '192.168.1.0/24'
    """
    if not int(net.hostmask):
        return str(net.network_address)
    return str(net)


class IPEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, (IPv4Network, IPv6Network)):
            return ip_net_to_string(obj)
        if isinstance(obj, (IPv4Address, IPv6Address)):
            return str(obj)
        return json.JSONEncoder.default(self, obj)


class ServerJSONEncoder(IPEncoder):
    def default(self, obj):
        if isinstance(obj, Model):
            return model_to_dict(obj)
        return super().default(obj)
