"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>
"""
import os

import yaml

from defence360agent.contracts.config import IConfigFile, LocalConfig
from defence360agent.utils import log_error_and_ignore


@log_error_and_ignore()
def migrate(
    migrator,
    database,
    fake=False,
    config_file: IConfigFile = LocalConfig(),
    **kwargs
):
    if fake:
        return

    if not os.path.exists(config_file.path):
        return

    with open(config_file.path) as f:
        conf = yaml.safe_load(f)

    malware_settings = conf.setdefault("MALWARE_SCANNING", {})
    value = malware_settings.pop("scan_modified_files", None)
    malware_settings["scan_modified_files"] = value

    config_file.dict_to_config(conf, validate=False)


def rollback(migrator, database, fake=False, **kwargs):
    pass
