"""
This program is free software: you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, 
or (at your option) any later version.


This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.


You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.

Copyright © 2019 Cloud Linux Software Inc.

This software is also available under ImunifyAV commercial license,
see <https://www.imunify360.com/legal/eula>
"""
import logging
import shutil

from defence360agent.files import FILES_DIR
from defence360agent.utils import antivirus_mode

logger = logging.getLogger(__name__)


def _move(src, dst):
    try:
        shutil.move(src, dst)
    except FileNotFoundError:
        pass
    except Exception as err:
        logger.error('Failed to move HackerTrap list to the new location: %r',
                     err)


@antivirus_mode.skip
def migrate(migrator, database, fake=False, **kwargs):
    if fake:
        return

    try:
        from defence360agent.contracts.config import HackerTrap
        from defence360agent.malwarelib.subsys.malware import \
            HackerTrapHitsSaver
    except ImportError:
        return

    HackerTrapHitsSaver.BASE_DIR = str(FILES_DIR)

    src1 = HackerTrapHitsSaver._filepath()
    src2 = HackerTrapHitsSaver._clean_filepath()
    for src in src1, src2:
        _move(str(src), HackerTrap.DIR)


def rollback(migrator, database, fake=False, **kwargs):
    pass
